//
// Simple torus class
//

#include    "Vector4D.h"
#include    "Vector3D.h"
#include    "Vector2D.h"

#ifndef __TORUS__
#define __TORUS__

struct  Vertex
{
    Vector4D    pos;                        // position of vertex
    Vector2D    tex;                        // texture coordinates
    Vector3D    n;                          // unit normal
    Vector3D    t, b;                       // tangent and binormal
    Vector4D    inf;                        // projection into infinity
};

struct  Face                                // triangular face
{
    int         index [3];                  // indices to Vertex array
    Vector3D    n;                          // face normal
    bool        frontFacing;
};

struct  Edge
{
    int     a, b;                           // indices into array of Vertex
    int     f1, f2;                         // indices into array of Face's
    bool    rev1, rev2;                     // whether the order of points was reversed
    										// for faces f1 and f2
};

class   Torus
{
    int      numRings;
    int      numSides;
    int      numVertices;
    int      numFaces;
    int      numEdges;
    Vertex * vertices;
    Face   * faces;
    Edge   * edges;

public:
    Torus  ( float r1, float r2, int rings, int sides );
    ~Torus ()
    {
        delete [] vertices;
        delete [] faces;
        delete [] edges;
    }

    int getNumVertices () const
    {
        return numVertices;
    }

    int getNumFaces () const
    {
        return numFaces;
    }

    int getNumEdges () const
    {
        return numEdges;
    }

    Vertex * getVertices ()
    {
        return vertices;
    }

    Face * getFaces ()
    {
        return faces;
    }

    Edge * getEdges ()
    {
        return edges;
    }

    void    draw      ();
    void    drawUnlit ();

protected:
    void    addEdge ( int a, int b, int face );

    int     indexFor ( int i, int j ) const
    {
        return (i % numRings) * numSides + (j % numSides);
    }
};

#endif
